
// Toroidal coil
// only a cross section is represented
// cylindrical symmetry around (OY) is assumed

Include "torus.par" ;

Group {
	B_ext = Region[64] ; 
	OY = Region[65] ;
	CoilSection = Region[63] ;
	Coil = Region[79] ;

	D = Region[66] ;
	D_inf = Region[67] ;
	D_tot = Region[{D,D_inf}] ;
}

N = 1000 ;	// nb of windings 
I = 1 ; 	// Current in the wire (A)

Function {
	mu0 = 4*Pi*1e-7 ;
	j_s[CoilSection] = N*I / (2*Pi*X[]) ;	// surface current on the coil
					// X[] = radial distance
	J_s[CoilSection] = j_s[] * Tangent[] ;  // vector

	j_s[Coil] = N*I / (2*Pi*Norm[Vector[X[],0,Z[]]]) ;
	J_s[Coil] = j_s[] * Unit[Cross[Normal[],Vector[-Z[],0,X[]]]] ;
	
}

/*
Conservation of the current implies that j_s behaves as 1/X.
One can try something else to see the difference.
*/

Jacobian {
  { Name JVol ;
    Case { 
        { Region D_inf ; Jacobian VolAxiSphShell {R_int, R_ext} ; }
        { Region All ; Jacobian VolAxi ; }
    }
  }
  { Name JSurAxi ;
    Case { 
      { Region CoilSection ; Jacobian SurAxi ; }
    }
  }
  { Name JSur ;
    Case {
      { Region Coil ; Jacobian Sur ; }
    }
  }

}


Integration {
  { Name I ;
    Case { 
      { Type Gauss ;
        Case { 
          { GeoElement Line        ; NumberOfPoints  3 ; }
          { GeoElement Triangle    ; NumberOfPoints  4 ; }
          { GeoElement Quadrangle  ; NumberOfPoints  4 ; }
        }
      }
    }
  }
}

Constraint {

	{ Name gauge_condition ;
	  Case {
	    { Region D_tot ; Type Assign ; Value 0 ; }
	  }
        }

}


FunctionSpace {
        { Name potentiel_vecteur_jauge ; Type Form1 ;
          BasisFunction {
             { Name w_n ; NameOfCoef a_n ; Function BF_Edge ;
               Support Region[{D_tot,CoilSection}] ; Entity EdgesOf[ All ] ; }
          }
          Constraint {
		{ NameOfCoef a_n ; EntityType EdgesOfTreeIn ;
                  NameOfConstraint gauge_condition ; } 
          }
        }

        { Name potentiel_vecteur_integrale ; Type Form1 ;
          BasisFunction {
             { Name w_n ; NameOfCoef a_n ; Function BF_Edge ;
               Support D_tot ; Entity EdgesOf[ All ] ; }
          }
        }

}


Formulation {
  { Name formulation_du_probleme ; Type FemEquation ;
    Quantity {
      { Name a1 ; Type Local  ; NameOfSpace potentiel_vecteur_jauge ; }

      { Name a2 ; Type Integral ; NameOfSpace potentiel_vecteur_integrale ;

//	[ Vector[0,0,1] ] ;    			// works as expected
//	[ j_s[] * Vector[0,0,1] ];		// several "nan" and "inf" ! 
	[ mu0 *  Laplace[]{3D} * J_s[] ];	// does not work at all

        In Coil ; Jacobian JSur ; Integration I ; }
    }

    Equation {

      Galerkin { [ 1/mu0 * Dof{Curl a1} , {Curl a1} ] ;
             In D_tot ; Jacobian JVol ; Integration I ; }
      
      Galerkin { [ - J_s[] , {a1} ] ;
             In CoilSection ; Jacobian JSurAxi ; Integration I ; }
    }
  }
}


Resolution {
  { Name resolution_du_lineaire ;
    System {
      { Name A ; NameOfFormulation formulation_du_probleme ; }
    }
    Operation { 
      InitSolution[A];
      Generate[A] ; 
      Solve[A] ; 
      SaveSolution[A] ; 
    }
  }
}

PostProcessing {
  { Name post_processing ; NameOfFormulation formulation_du_probleme ;
    Quantity {
     { Name a1 ; Value { Local { [ {a1} ] ; In D_tot ; Jacobian JVol ; }}}
     { Name B ; Value { Local { [ {Curl a1} ] ; In D_tot ; Jacobian JVol ; }}}
     { Name a2 ; Value { Local { [ {a2} ] ; 
       In D_tot ; Jacobian JVol ; }}}
     { Name J_s ; Value { Local { [ J_s[] ] ; In Coil ; Jacobian JSur ; }}}

    }
  }
}

PostOperation {
  { Name post_operations ; NameOfPostProcessing post_processing;
    Operation {
        Print[ a1, OnElementsOf D_tot, File "a1.pos" ] ;
        Print[ B, OnElementsOf D_tot, File "B.pos" ] ;
        Print[ a2, OnElementsOf D_tot, File "a2.pos" ] ;
        Print[ J_s, OnElementsOf Coil, File "J_s.pos" ] ;
    }
  }
}


